/*
 * System Interface Library for games
 * Copyright (c) 2007-2014 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/main.c: Global initialization and cleanup functions.
 */

#include "src/base.h"
#include "src/debug.h"
#include "src/graphics.h"
#include "src/input.h"
#include "src/main.h"
#include "src/resource.h"
#include "src/sound.h"
#include "src/test.h"
#include "src/time.h"

#undef sil__main  // Disable the wrapper in main.h.

/*************************************************************************/
/*************************************************************************/

int init_all(void)
{
    /* Call time_init() first because other functions may need to call the
     * timing routines. */
    time_init();

#ifdef DEBUG
    debug_init();
#endif
    if (graphics_init()) {
        if (input_init()) {
            resource_init();
            sound_init();
            return 1;
        } else {
            DLOG("input_init() failed!");
        }
        graphics_cleanup();
    } else {
        DLOG("graphics_init() failed!");
    }
#ifdef DEBUG
    debug_cleanup();
#endif
    return 0;
}

/*-----------------------------------------------------------------------*/

void cleanup_all(void)
{
#ifdef DEBUG
    debug_cleanup();
#endif
    graphics_cleanup();
    input_cleanup();
    resource_cleanup();
    sound_cleanup();
}

/*-----------------------------------------------------------------------*/

int sil__main(int argc, const char **argv, int (*p_main)(int, const char **))
{
    if (!init_all()) {
        return 2;
    }
    int exit_code = (*p_main)(argc, argv);
    ASSERT(exit_code == EXIT_SUCCESS || exit_code == EXIT_FAILURE,
           exit_code = EXIT_FAILURE);
    cleanup_all();
    return exit_code == EXIT_SUCCESS ? 0 : 1;
}

/*************************************************************************/
/*************************************************************************/

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
