/*
 * System Interface Library for games
 * Copyright (c) 2007-2014 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/sysdep/openal/openal.h: Header for exported OpenAL routines.
 */

#ifndef SIL_SRC_SYSDEP_OPENAL_OPENAL_H
#define SIL_SRC_SYSDEP_OPENAL_OPENAL_H

/*************************************************************************/
/*************************************************************************/

/**
 * openal_init:  Initialize the OpenAL subsystem.  This function implements
 * all OpenAL-specific behavior for sys_sound_init(); if no additional
 * platform-dependent code is required, sys_sound_init() can simply be a
 * trampoline to this function.
 *
 * [Parameters]
 *     device_name: System-dependent device name for audio output.
 *     num_channels: Number of playback channels to allocate.
 *     channels_ret: Array into which channel IDs are stored.
 * [Return value]
 *     Number of channels available for playback, or zero on error.
 */
extern int openal_init(const char *device_name, int num_channels,
                       int *channels_ret);

/**
 * openal_cleanup:  Shut down the OpenAL subsystem.  This function
 * implements all OpenAL-specific behavior for sys_sound_cleanup().
 */
extern void openal_cleanup(void);

/**
 * openal_suspend, openal_resume:  Suspend or resume low-level audio output.
 */
extern void openal_suspend(void);
extern void openal_resume(void);

/*************************************************************************/
/*************************************************************************/

#endif  // SIL_SRC_SYSDEP_OPENAL_OPENAL_H

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
