/*
 * System Interface Library for games
 * Copyright (c) 2007-2014 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/sysdep/windows/main.c: Program entry point for Windows.
 */

#define IN_SYSDEP

#include "src/base.h"
#include "src/main.h"
#include "src/math/fpu.h"
#include "src/memory.h"
#include "src/sysdep.h"
#include "src/sysdep/windows/internal.h"
#include "src/utility/misc.h"

#ifdef __MINGW32__  // See below.
# include <stdio.h>
#endif

/*************************************************************************/
/************************** Program entry point **************************/
/*************************************************************************/

int WINAPI WinMain(UNUSED HINSTANCE hInstance, UNUSED HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, UNUSED int nCmdShow)
{
#ifdef __MINGW32__
    /* MinGW and/or MSYS seem to be unaware that stderr is supposed to be
     * _un_buffered... */
    setbuf(stderr, NULL);
#endif

#if defined(_MSC_VER) && defined(_DEBUG)
    _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_DEBUG);
#endif

    fpu_configure();

    HRESULT res = CoInitialize(NULL);
    if (UNLIKELY(res != S_OK)) {
        DLOG("CoInitialize() failed: %s", windows_strerror(res));
        goto fail;
    }

    if (UNLIKELY(!windows_file_init()) || UNLIKELY(!windows_userdata_init())) {
        goto fail;
    }

    int argc;
    const char **argv;
    const char *argv_dummy[1];
    if (UNLIKELY(!split_args(lpCmdLine, 1, &argc, (char ***)&argv))) {
        argc = 1;
        argv = argv_dummy;
        argv[0] = "SIL";
    }

    const int exitcode = sil__main(argc, argv);
    if (UNLIKELY(exitcode == 2)) {
        goto fail;
    }

    if (argv != argv_dummy) {
        mem_free(argv);
    }

    return exitcode;

  fail:
    display_error("An unrecoverable error occurred while starting the"
                  " program.");
    return 1;
}

/*************************************************************************/
/*************************************************************************/

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
